@@ -1,6 +1,4 @@
-from __future__ import print_function, division
-
-from sympy.combinatorics.permutations import Permutation, _af_rmul, _af_rmuln,\
+from sympy.combinatorics.permutations import Permutation, _af_rmul, \
     _af_invert, _af_new
 from sympy.combinatorics.perm_groups import PermutationGroup, _orbit, \
     _orbit_transversal
@@ -48,7 +46,7 @@ def dummy_sgs(dummies, sym, n):
     ========
 
     >>> from sympy.combinatorics.tensor_can import dummy_sgs
-    >>> dummy_sgs(range(2, 8), 0, 8)
+    >>> dummy_sgs(list(range(2, 8)), 0, 8)
     [[0, 1, 3, 2, 4, 5, 6, 7, 8, 9], [0, 1, 2, 3, 5, 4, 6, 7, 8, 9],
      [0, 1, 2, 3, 4, 5, 7, 6, 8, 9], [0, 1, 4, 5, 2, 3, 6, 7, 8, 9],
      [0, 1, 2, 3, 6, 7, 4, 5, 8, 9]]
@@ -167,6 +165,9 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     """
     Butler-Portugal algorithm for tensor canonicalization with dummy indices
 
+    Parameters
+    ==========
+
       dummies
         list of lists of dummy indices,
         one list for each type of index;
@@ -193,9 +194,14 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
       g
         permutation representing the tensor.
 
+    Returns
+    =======
+
     Return 0 if the tensor is zero, else return the array form of
     the permutation representing the canonical form of the tensor.
 
+    Notes
+    =====
 
     A tensor with dummy indices can be represented in a number
     of equivalent ways which typically grows exponentially with
@@ -221,7 +227,7 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     (the indices including both the contravariant and the covariant ones)
     can be written as
 
-    `t = T(ind[g[0],..., ind[g[n-1]])`,
+    `t = T(ind[g[0]],..., ind[g[n-1]])`,
 
     where `n= len(ind)`;
     `g` has size `n + 2`, the last two indices for the sign of the tensor
@@ -237,7 +243,8 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     these symmetries, one can represent the tensor by `g`, which transforms
     as
 
-    `g -> d*g*s`, so it belongs to the coset `D*g*S`.
+    `g -> d*g*s`, so it belongs to the coset `D*g*S`, or in other words
+    to the set of all permutations allowed by the slot and dummy symmetries.
 
     Let us explain the conventions by an example.
 
@@ -248,51 +255,54 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
 
     and symmetric metric, find the tensor equivalent to it which
     is the lowest under the ordering of indices:
-    lexicographic ordering `d1, d2, d3` then and contravariant index
+    lexicographic ordering `d1, d2, d3` and then contravariant
     before covariant index; that is the canonical form of the tensor.
 
     The canonical form is `-T^{d1 d2 d3}{}_{d1 d2 d3}`
     obtained using `T^{a0 a1 a2 a3 a4 a5} = -T^{a2 a1 a0 a3 a4 a5}`.
 
     To convert this problem in the input for this function,
-    use the following labelling of the index names
+    use the following ordering of the index names
     (- for covariant for short) `d1, -d1, d2, -d2, d3, -d3`
 
-    `T^{d3 d2 d1}{}_{d1 d2 d3}` corresponds to `g = [4,2,0,1,3,5,6,7]`
+    `T^{d3 d2 d1}{}_{d1 d2 d3}` corresponds to `g = [4, 2, 0, 1, 3, 5, 6, 7]`
     where the last two indices are for the sign
 
-    `sgens = [Permutation(0,2)(6,7), Permutation(0,4)(6,7)]`
+    `sgens = [Permutation(0, 2)(6, 7), Permutation(0, 4)(6, 7)]`
 
-    sgens[0] is the slot symmetry `-(0,2)`
+    sgens[0] is the slot symmetry `-(0, 2)`
     `T^{a0 a1 a2 a3 a4 a5} = -T^{a2 a1 a0 a3 a4 a5}`
 
-    sgens[1] is the slot symmetry `-(0,4)`
+    sgens[1] is the slot symmetry `-(0, 4)`
     `T^{a0 a1 a2 a3 a4 a5} = -T^{a4 a1 a2 a3 a0 a5}`
 
     The dummy symmetry group D is generated by the strong base generators
-    `[(0,1),(2,3),(4,5),(0,1)(2,3),(2,3)(4,5)]`
+    `[(0, 1), (2, 3), (4, 5), (0, 2)(1, 3), (0, 4)(1, 5)]`
+    where the first three interchange covariant and contravariant
+    positions of the same index (d1 <-> -d1) and the last two interchange
+    the dummy indices themselves (d1 <-> d2).
 
     The dummy symmetry acts from the left
-    `d = [1,0,2,3,4,5,6,7]`  exchange `d1 -> -d1`
+    `d = [1, 0, 2, 3, 4, 5, 6, 7]`  exchange `d1 <-> -d1`
     `T^{d3 d2 d1}{}_{d1 d2 d3} == T^{d3 d2}{}_{d1}{}^{d1}{}_{d2 d3}`
 
-    `g=[4,2,0,1,3,5,6,7]  -> [4,2,1,0,3,5,6,7] = _af_rmul(d, g)`
+    `g=[4, 2, 0, 1, 3, 5, 6, 7]  -> [4, 2, 1, 0, 3, 5, 6, 7] = _af_rmul(d, g)`
     which differs from `_af_rmul(g, d)`.
 
     The slot symmetry acts from the right
-    `s = [2,1,0,3,4,5,7,6]`  exchanges slots 0 and 2 and changes sign
+    `s = [2, 1, 0, 3, 4, 5, 7, 6]`  exchanges slots 0 and 2 and changes sign
     `T^{d3 d2 d1}{}_{d1 d2 d3} == -T^{d1 d2 d3}{}_{d1 d2 d3}`
 
-    `g=[4,2,0,1,3,5,6,7]  -> [0,2,4,1,3,5,7,6] = _af_rmul(g, s)`
+    `g=[4,2,0,1,3,5,6,7]  -> [0, 2, 4, 1, 3, 5, 7, 6] = _af_rmul(g, s)`
 
     Example in which the tensor is zero, same slot symmetries as above:
-    `T^{d3}{}_{d1,d2}{}^{d1}{}_{d3}{}^{d2}`
+    `T^{d2}{}_{d1 d3}{}^{d1 d3}{}_{d2}`
 
-    `= -T^{d3}{}_{d1,d3}{}^{d1}{}_{d2}{}^{d2}`   under slot symmetry `-(2,4)`;
+    `= -T^{d3}{}_{d1 d3}{}^{d1 d2}{}_{d2}`   under slot symmetry `-(0,4)`;
 
-    `= T_{d3 d1}{}^{d3}{}^{d1}{}_{d2}{}^{d2}`    under slot symmetry `-(0,2)`;
+    `= T_{d3 d1}{}^{d3}{}^{d1 d2}{}_{d2}`    under slot symmetry `-(0,2)`;
 
-    `= T^{d3}{}_{d1 d3}{}^{d1}{}_{d2}{}^{d2}`    symmetric metric;
+    `= T^{d3}{}_{d1 d3}{}^{d1 d2}{}_{d2}`    symmetric metric;
 
     `= 0`  since two of these lines have tensors differ only for the sign.
 
@@ -317,7 +327,7 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     the stabilizers of the strong base generators are the
     strong base generators of the stabilizer subgroup.
 
-    `dbase = [p0,p1,...]` is not in general in lexicographic order,
+    `dbase = [p0, p1, ...]` is not in general in lexicographic order,
     so that one must recompute the strong base generators each time;
     however this is trivial, there is no need to use the Schreier-Sims
     algorithm for D.
@@ -379,14 +389,14 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     >>> from sympy.combinatorics.permutations import Permutation
     >>> from sympy.combinatorics.perm_groups import PermutationGroup
     >>> from sympy.combinatorics.tensor_can import double_coset_can_rep, get_transversals
-    >>> gens = [Permutation(x) for x in [[2,1,0,3,4,5,7,6], [4,1,2,3,0,5,7,6]]]
+    >>> gens = [Permutation(x) for x in [[2, 1, 0, 3, 4, 5, 7, 6], [4, 1, 2, 3, 0, 5, 7, 6]]]
     >>> base = [0, 2]
-    >>> g = Permutation([4,2,0,1,3,5,6,7])
+    >>> g = Permutation([4, 2, 0, 1, 3, 5, 6, 7])
     >>> transversals = get_transversals(base, gens)
     >>> double_coset_can_rep([list(range(6))], [0], base, gens, transversals, g)
     [0, 1, 2, 3, 4, 5, 7, 6]
 
-    >>> g = Permutation([4,1,3,0,5,2,6,7])
+    >>> g = Permutation([4, 1, 3, 0, 5, 2, 6, 7])
     >>> double_coset_can_rep([list(range(6))], [0], base, gens, transversals, g)
     0
     """
@@ -408,7 +418,6 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     dsgsx = []
     for i in range(num_types):
         dsgsx.extend(dummy_sgs(dumx[i], sym[i], num_dummies))
-    ginv = _af_invert(g)
     idn = list(range(size))
     # TAB = list of entries (s, d, h) where h = _af_rmuln(d,g,s)
     # for short, in the following d*g*s means _af_rmuln(d,g,s)
@@ -420,7 +429,7 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
             sgensx1 = [_af_new(_) for _ in sgensx]
             deltab = _orbit(size, sgensx1, b)
         else:
-            deltab = set([b])
+            deltab = {b}
         # p1 = min(IMAGES) = min(Union D_p*h*deltab for h in TAB)
         if all_metrics_with_sym:
             md = _min_dummies(dumx, sym, indices)
@@ -449,7 +458,6 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
         else:
             deltap = [p_i]
         TAB1 = []
-        nTAB = len(TAB)
         while TAB:
             s, d, h = TAB.pop()
             if min([md[h[x]] for x in deltab]) != p_i:
@@ -483,7 +491,7 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
                         s1 = [s[ix] for ix in s1]
                 else:
                     s1 = s
-                #assert s1[b] == j  # invariant
+                # assert s1[b] == j  # invariant
                 # solve d1*g*j = p_i with d1 = dx*d for some element dg
                 # of the stabilizer of ..., p_{i-1}
                 # dx**-1*p_i = d*g*j; dx**-1 = trace_D(d*g*j,...)
@@ -501,13 +509,12 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
                 assert d1[dg[j]] == p_i  # invariant
                 d1 = [d1[ix] for ix in d]
                 h1 = [d1[g[ix]] for ix in s1]
-                #assert h1[b] == p_i  # invariant
+                # assert h1[b] == p_i  # invariant
                 TAB1.append((s1, d1, h1))
 
         # if TAB contains equal permutations, keep only one of them;
         # if TAB contains equal permutations up to the sign, return 0
         TAB1.sort(key=lambda x: x[-1])
-        nTAB1 = len(TAB1)
         prev = [0] * size
         while TAB1:
             s, d, h = TAB1.pop()
@@ -548,7 +555,7 @@ def canonical_free(base, gens, g, num_free):
 
     >>> from sympy.combinatorics import Permutation
     >>> from sympy.combinatorics.tensor_can import canonical_free
-    >>> gens = [[1,0,2,3,5,4], [2,3,0,1,4,5],[0,1,3,2,5,4]]
+    >>> gens = [[1, 0, 2, 3, 5, 4], [2, 3, 0, 1, 4, 5],[0, 1, 3, 2, 5, 4]]
     >>> gens = [Permutation(h) for h in gens]
     >>> base = [0, 2]
     >>> g = Permutation([2, 1, 0, 3, 4, 5])
@@ -557,9 +564,9 @@ def canonical_free(base, gens, g, num_free):
 
     Consider the product of Riemann tensors
     ``T = R^{a}_{d0}^{d1,d2}*R_{d2,d1}^{d0,b}``
-    The order of the indices is ``[a,b,d0,-d0,d1,-d1,d2,-d2]``
+    The order of the indices is ``[a, b, d0, -d0, d1, -d1, d2, -d2]``
     The permutation corresponding to the tensor is
-    ``g = [0,3,4,6,7,5,2,1,8,9]``
+    ``g = [0, 3, 4, 6, 7, 5, 2, 1, 8, 9]``
 
     In particular ``a`` is position ``0``, ``b`` is in position ``9``.
     Use the slot symmetries to get `T` is a form which is the minimal
@@ -569,8 +576,8 @@ def canonical_free(base, gens, g, num_free):
 
     >>> from sympy.combinatorics.tensor_can import riemann_bsgs, tensor_gens
     >>> base, gens = riemann_bsgs
-    >>> size, sbase, sgens = tensor_gens(base, gens, [[],[]], 0)
-    >>> g = Permutation([0,3,4,6,7,5,2,1,8,9])
+    >>> size, sbase, sgens = tensor_gens(base, gens, [[], []], 0)
+    >>> g = Permutation([0, 3, 4, 6, 7, 5, 2, 1, 8, 9])
     >>> canonical_free(sbase, [Permutation(h) for h in sgens], g, 2)
     [0, 3, 4, 6, 1, 2, 7, 5, 9, 8]
     """
@@ -580,13 +587,11 @@ def canonical_free(base, gens, g, num_free):
         return g[:]
 
     transversals = get_transversals(base, gens)
-    m = len(base)
     for x in sorted(g[:-2]):
         if x not in base:
             base.append(x)
     h = g
     for i, transv in enumerate(transversals):
-        b = base[i]
         h_i = [size]*num_free
         # find the element s in transversals[i] such that
         # _af_rmul(h, s) has its free elements with the lowest position in h
@@ -701,7 +706,7 @@ def canonicalize(g, dummies, msym, *v):
 
     `ord = [d0,-d0,d1,-d1,d2,-d2]` order of the indices
 
-    g = [1,3,0,5,4,2,6,7]
+    g = [1, 3, 0, 5, 4, 2, 6, 7]
 
     `T_c = 0`
 
@@ -710,7 +715,7 @@ def canonicalize(g, dummies, msym, *v):
     >>> base2a, gens2a = get_symmetric_group_sgs(2, 1)
     >>> t0 = (base2a, gens2a, 1, 0)
     >>> t1 = (base2a, gens2a, 2, 0)
-    >>> g = Permutation([1,3,0,5,4,2,6,7])
+    >>> g = Permutation([1, 3, 0, 5, 4, 2, 6, 7])
     >>> canonicalize(g, range(6), 0, t0, t1)
     0
 
@@ -748,7 +753,7 @@ def canonicalize(g, dummies, msym, *v):
     >>> t0 = (base_f, gens_f, 2, 0)
     >>> t1 = (base_A, gens_A, 4, 0)
     >>> dummies = [range(2, 10), range(10, 14)]
-    >>> g = Permutation([0,7,3,1,9,5,11,6,10,4,13,2,12,8,14,15])
+    >>> g = Permutation([0, 7, 3, 1, 9, 5, 11, 6, 10, 4, 13, 2, 12, 8, 14, 15])
     >>> canonicalize(g, dummies, [0, 0], t0, t1)
     [0, 2, 4, 1, 6, 8, 10, 3, 11, 7, 12, 5, 13, 9, 15, 14]
     """
@@ -831,7 +836,6 @@ def canonicalize(g, dummies, msym, *v):
     size, sbase, sgens = gens_products(*v1)
 
     # reduce the permutations getting rid of the free indices
-    pos_dummies = [g1.index(x) for x in flat_dummies]
     pos_free = [g1.index(x) for x in range(num_free)]
     size_red = size - num_free
     g1_red = [x - num_free for x in g1 if x in flat_dummies]
@@ -862,12 +866,12 @@ def perm_af_direct_product(gens1, gens2, signed=True):
     ========
 
     >>> from sympy.combinatorics.tensor_can import perm_af_direct_product
-    >>> gens1 = [[1,0,2,3], [0,1,3,2]]
-    >>> gens2 = [[1,0]]
+    >>> gens1 = [[1, 0, 2, 3], [0, 1, 3, 2]]
+    >>> gens2 = [[1, 0]]
     >>> perm_af_direct_product(gens1, gens2, False)
     [[1, 0, 2, 3, 4, 5], [0, 1, 3, 2, 4, 5], [0, 1, 2, 3, 5, 4]]
-    >>> gens1 = [[1,0,2,3,5,4], [0,1,3,2,4,5]]
-    >>> gens2 = [[1,0,2,3]]
+    >>> gens1 = [[1, 0, 2, 3, 5, 4], [0, 1, 3, 2, 4, 5]]
+    >>> gens2 = [[1, 0, 2, 3]]
     >>> perm_af_direct_product(gens1, gens2, True)
     [[1, 0, 2, 3, 4, 5, 7, 6], [0, 1, 3, 2, 4, 5, 6, 7], [0, 1, 2, 3, 5, 4, 6, 7]]
     """
@@ -893,7 +897,10 @@ def perm_af_direct_product(gens1, gens2, signed=True):
 
 def bsgs_direct_product(base1, gens1, base2, gens2, signed=True):
     """
-    direct product of two BSGS
+    Direct product of two BSGS
+
+    Parameters
+    ==========
 
     base1    base of the first BSGS.
 
@@ -908,11 +915,10 @@ def bsgs_direct_product(base1, gens1, base2, gens2, signed=True):
 
     >>> from sympy.combinatorics import Permutation
     >>> from sympy.combinatorics.tensor_can import (get_symmetric_group_sgs, bsgs_direct_product)
-    >>> Permutation.print_cyclic = True
     >>> base1, gens1 = get_symmetric_group_sgs(1)
     >>> base2, gens2 = get_symmetric_group_sgs(2)
     >>> bsgs_direct_product(base1, gens1, base2, gens2)
-    ([1], [Permutation(4)(1, 2)])
+    ([1], [(4)(1 2)])
     """
     s = 2 if signed else 0
     n1 = gens1[0].size - s
@@ -943,9 +949,8 @@ def get_symmetric_group_sgs(n, antisym=False):
 
     >>> from sympy.combinatorics import Permutation
     >>> from sympy.combinatorics.tensor_can import get_symmetric_group_sgs
-    >>> Permutation.print_cyclic = True
     >>> get_symmetric_group_sgs(3)
-    ([0, 1], [Permutation(4)(0, 1), Permutation(4)(1, 2)])
+    ([0, 1], [(4)(0 1), (4)(1 2)])
     """
     if n == 1:
         return [], [_af_new(list(range(3)))]
@@ -969,7 +974,7 @@ def get_transversals(base, gens):
         return []
     stabs = _distribute_gens_by_base(base, gens)
     orbits, transversals = _orbits_transversals_from_bsgs(base, stabs)
-    transversals = [dict((x, h._array_form) for x, h in y.items()) for y in
+    transversals = [{x: h._array_form for x, h in y.items()} for y in
                     transversals]
     return transversals
 
@@ -987,7 +992,7 @@ def _is_minimal_bsgs(base, gens):
     >>> from sympy.combinatorics.tensor_can import riemann_bsgs, _is_minimal_bsgs
     >>> _is_minimal_bsgs(*riemann_bsgs)
     True
-    >>> riemann_bsgs1 = ([2, 0], ([Permutation(5)(0,1)(4,5), Permutation(5)(0,2)(1,3)]))
+    >>> riemann_bsgs1 = ([2, 0], ([Permutation(5)(0, 1)(4, 5), Permutation(5)(0, 2)(1, 3)]))
     >>> _is_minimal_bsgs(*riemann_bsgs1)
     False
     """
@@ -1018,10 +1023,9 @@ def get_minimal_bsgs(base, gens):
 
     >>> from sympy.combinatorics import Permutation
     >>> from sympy.combinatorics.tensor_can import get_minimal_bsgs
-    >>> Permutation.print_cyclic = True
-    >>> riemann_bsgs1 = ([2, 0], ([Permutation(5)(0,1)(4,5), Permutation(5)(0,2)(1,3)]))
+    >>> riemann_bsgs1 = ([2, 0], ([Permutation(5)(0, 1)(4, 5), Permutation(5)(0, 2)(1, 3)]))
     >>> get_minimal_bsgs(*riemann_bsgs1)
-    ([0, 2], [Permutation(0, 1)(4, 5), Permutation(5)(0, 2)(1, 3), Permutation(2, 3)(4, 5)])
+    ([0, 2], [(0 1)(4 5), (5)(0 2)(1 3), (2 3)(4 5)])
     """
     G = PermutationGroup(gens)
     base, gens = G.schreier_sims_incremental()
@@ -1032,7 +1036,8 @@ def get_minimal_bsgs(base, gens):
 
 def tensor_gens(base, gens, list_free_indices, sym=0):
     """
-    Returns size, res_base, res_gens BSGS for n tensors of the same type
+    Returns size, res_base, res_gens BSGS for n tensors of the
+    same type
 
     base, gens BSGS for tensors of this type
     list_free_indices  list of the slots occupied by fixed indices
@@ -1048,16 +1053,17 @@ def tensor_gens(base, gens, list_free_indices, sym=0):
 
     >>> from sympy.combinatorics import Permutation
     >>> from sympy.combinatorics.tensor_can import tensor_gens, get_symmetric_group_sgs
-    >>> Permutation.print_cyclic = True
 
     two symmetric tensors with 3 indices without free indices
+
     >>> base, gens = get_symmetric_group_sgs(3)
     >>> tensor_gens(base, gens, [[], []])
-    (8, [0, 1, 3, 4], [Permutation(7)(0, 1), Permutation(7)(1, 2), Permutation(7)(3, 4), Permutation(7)(4, 5), Permutation(7)(0, 3)(1, 4)(2, 5)])
+    (8, [0, 1, 3, 4], [(7)(0 1), (7)(1 2), (7)(3 4), (7)(4 5), (7)(0 3)(1 4)(2 5)])
 
     two symmetric tensors with 3 indices with free indices in slot 1 and 0
-    >>> tensor_gens(base, gens, [[1],[0]])
-    (8, [0, 4], [Permutation(7)(0, 2), Permutation(7)(4, 5)])
+
+    >>> tensor_gens(base, gens, [[1], [0]])
+    (8, [0, 4], [(7)(0 2), (7)(4 5)])
 
     four symmetric tensors with 3 indices, two of which with free indices
 
@@ -1163,12 +1169,11 @@ def gens_products(*v):
 
     >>> from sympy.combinatorics import Permutation
     >>> from sympy.combinatorics.tensor_can import get_symmetric_group_sgs, gens_products
-    >>> Permutation.print_cyclic = True
     >>> base, gens = get_symmetric_group_sgs(2)
-    >>> gens_products((base,gens,[[],[]],0))
-    (6, [0, 2], [Permutation(5)(0, 1), Permutation(5)(2, 3), Permutation(5)(0, 2)(1, 3)])
-    >>> gens_products((base,gens,[[1],[]],0))
-    (6, [2], [Permutation(5)(2, 3)])
+    >>> gens_products((base, gens, [[], []], 0))
+    (6, [0, 2], [(5)(0 1), (5)(2 3), (5)(0 2)(1 3)])
+    >>> gens_products((base, gens, [[1], []], 0))
+    (6, [2], [(5)(2 3)])
     """
     res_size, res_base, res_gens = tensor_gens(*v[0])
     for i in range(1, len(v)):
