@@ -3,6 +3,8 @@
 TODO:
 * Address Issue 2251, printing of spin states
 """
+from typing import Dict, Any
+
 from sympy.physics.quantum.anticommutator import AntiCommutator
 from sympy.physics.quantum.cg import CG, Wigner3j, Wigner6j, Wigner9j
 from sympy.physics.quantum.commutator import Commutator
@@ -21,13 +23,10 @@ from sympy.physics.quantum.sho1d import RaisingOp
 
 from sympy import Derivative, Function, Interval, Matrix, Pow, S, symbols, Symbol, oo
 from sympy.core.compatibility import exec_
-from sympy.utilities.pytest import XFAIL
+from sympy.testing.pytest import XFAIL
 
 # Imports used in srepr strings
-from sympy.physics.quantum.constants import HBar
-from sympy.physics.quantum.hilbert import DirectSumHilbertSpace, TensorProductHilbertSpace, TensorPowerHilbertSpace
-from sympy.physics.quantum.spin import JzOp, J2Op
-from sympy import Add, Integer, Mul, Rational, Tuple
+from sympy.physics.quantum.spin import JzOp
 
 from sympy.printing import srepr
 from sympy.printing.pretty import pretty as xpretty
@@ -37,8 +36,17 @@ from sympy.core.compatibility import u_decode as u
 
 MutableDenseMatrix = Matrix
 
-ENV = {}
-exec_("from sympy import *", ENV)
+
+ENV = {}  # type: Dict[str, Any]
+exec_('from sympy import *', ENV)
+exec_('from sympy.physics.quantum import *', ENV)
+exec_('from sympy.physics.quantum.cg import *', ENV)
+exec_('from sympy.physics.quantum.spin import *', ENV)
+exec_('from sympy.physics.quantum.hilbert import *', ENV)
+exec_('from sympy.physics.quantum.qubit import *', ENV)
+exec_('from sympy.physics.quantum.qexpr import *', ENV)
+exec_('from sympy.physics.quantum.gate import *', ENV)
+exec_('from sympy.physics.quantum.constants import *', ENV)
 
 
 def sT(expr, string):
@@ -47,7 +55,7 @@ def sT(expr, string):
     from sympy/printing/tests/test_repr.py
     """
     assert srepr(expr) == string
-    assert eval(string) == expr
+    assert eval(string, ENV) == expr
 
 
 def pretty(expr):
@@ -67,7 +75,7 @@ def test_anticommutator():
     ac_tall = AntiCommutator(A**2, B)
     assert str(ac) == '{A,B}'
     assert pretty(ac) == '{A,B}'
-    assert upretty(ac) == u('{A,B}')
+    assert upretty(ac) == u'{A,B}'
     assert latex(ac) == r'\left\{A,B\right\}'
     sT(ac, "AntiCommutator(Operator(Symbol('A')),Operator(Symbol('B')))")
     assert str(ac_tall) == '{A**2,B}'
@@ -178,7 +186,7 @@ def test_commutator():
     c_tall = Commutator(A**2, B)
     assert str(c) == '[A,B]'
     assert pretty(c) == '[A,B]'
-    assert upretty(c) == u('[A,B]')
+    assert upretty(c) == u'[A,B]'
     assert latex(c) == r'\left[A,B\right]'
     sT(c, "Commutator(Operator(Symbol('A')),Operator(Symbol('B')))")
     assert str(c_tall) == '[A**2,B]'
@@ -201,7 +209,7 @@ u("""\
 def test_constants():
     assert str(hbar) == 'hbar'
     assert pretty(hbar) == 'hbar'
-    assert upretty(hbar) == u('ℏ')
+    assert upretty(hbar) == u'ℏ'
     assert latex(hbar) == r'\hbar'
     sT(hbar, "HBar()")
 
@@ -222,7 +230,7 @@ x \
 """)
     assert pretty(expr) == ascii_str
     assert upretty(expr) == ucode_str
-    assert latex(expr) == r'x^{\dag}'
+    assert latex(expr) == r'x^{\dagger}'
     sT(expr, "Dagger(Symbol('x'))")
 
 
@@ -244,7 +252,7 @@ def test_gate():
     g4 = UGate((0,), uMat)
     assert str(g1) == '1(2)'
     assert pretty(g1) == '1 \n 2'
-    assert upretty(g1) == u('1 \n 2')
+    assert upretty(g1) == u'1 \n 2'
     assert latex(g1) == r'1_{2}'
     sT(g1, "IdentityGate(Integer(2))")
     assert str(g1*q) == '1(2)*|10101>'
@@ -321,7 +329,7 @@ def test_hilbert():
     h4 = L2(Interval(0, oo))
     assert str(h1) == 'H'
     assert pretty(h1) == 'H'
-    assert upretty(h1) == u('H')
+    assert upretty(h1) == u'H'
     assert latex(h1) == r'\mathcal{H}'
     sT(h1, "HilbertSpace()")
     assert str(h2) == 'C(2)'
@@ -341,10 +349,10 @@ C \
     sT(h2, "ComplexSpace(Integer(2))")
     assert str(h3) == 'F'
     assert pretty(h3) == 'F'
-    assert upretty(h3) == u('F')
+    assert upretty(h3) == u'F'
     assert latex(h3) == r'\mathcal{F}'
     sT(h3, "FockSpace()")
-    assert str(h4) == 'L2([0, oo))'
+    assert str(h4) == 'L2(Interval(0, oo))'
     ascii_str = \
 """\
  2\n\
@@ -358,7 +366,7 @@ L \
     assert pretty(h4) == ascii_str
     assert upretty(h4) == ucode_str
     assert latex(h4) == r'{\mathcal{L}^2}\left( \left[0, \infty\right) \right)'
-    sT(h4, "L2(Interval(Integer(0), oo, S.false, S.true))")
+    sT(h4, "L2(Interval(Integer(0), oo, false, true))")
     assert str(h1 + h2) == 'H+C(2)'
     ascii_str = \
 """\
@@ -418,24 +426,24 @@ def test_innerproduct():
     ip_tall3 = InnerProduct(Bra(x/2), Ket(x))
     assert str(ip1) == '<psi|psi>'
     assert pretty(ip1) == '<psi|psi>'
-    assert upretty(ip1) == u('⟨ψ❘ψ⟩')
+    assert upretty(ip1) == u'⟨ψ❘ψ⟩'
     assert latex(
         ip1) == r'\left\langle \psi \right. {\left|\psi\right\rangle }'
     sT(ip1, "InnerProduct(Bra(Symbol('psi')),Ket(Symbol('psi')))")
     assert str(ip2) == '<psi;t|psi;t>'
     assert pretty(ip2) == '<psi;t|psi;t>'
-    assert upretty(ip2) == u('⟨ψ;t❘ψ;t⟩')
+    assert upretty(ip2) == u'⟨ψ;t❘ψ;t⟩'
     assert latex(ip2) == \
         r'\left\langle \psi;t \right. {\left|\psi;t\right\rangle }'
     sT(ip2, "InnerProduct(TimeDepBra(Symbol('psi'),Symbol('t')),TimeDepKet(Symbol('psi'),Symbol('t')))")
     assert str(ip3) == "<1,1|1,1>"
     assert pretty(ip3) == '<1,1|1,1>'
-    assert upretty(ip3) == u('⟨1,1❘1,1⟩')
+    assert upretty(ip3) == u'⟨1,1❘1,1⟩'
     assert latex(ip3) == r'\left\langle 1,1 \right. {\left|1,1\right\rangle }'
     sT(ip3, "InnerProduct(JzBra(Integer(1),Integer(1)),JzKet(Integer(1),Integer(1)))")
     assert str(ip4) == "<1,1,j1=1,j2=1|1,1,j1=1,j2=1>"
     assert pretty(ip4) == '<1,1,j1=1,j2=1|1,1,j1=1,j2=1>'
-    assert upretty(ip4) == u('⟨1,1,j₁=1,j₂=1❘1,1,j₁=1,j₂=1⟩')
+    assert upretty(ip4) == u'⟨1,1,j₁=1,j₂=1❘1,1,j₁=1,j₂=1⟩'
     assert latex(ip4) == \
         r'\left\langle 1,1,j_{1}=1,j_{2}=1 \right. {\left|1,1,j_{1}=1,j_{2}=1\right\rangle }'
     sT(ip4, "InnerProduct(JzBraCoupled(Integer(1),Integer(1),Tuple(Integer(1), Integer(1)),Tuple(Tuple(Integer(1), Integer(2), Integer(1)))),JzKetCoupled(Integer(1),Integer(1),Tuple(Integer(1), Integer(1)),Tuple(Tuple(Integer(1), Integer(2), Integer(1)))))")
@@ -505,7 +513,7 @@ u("""\
 
 def test_operator():
     a = Operator('A')
-    b = Operator('B', Symbol('t'), S(1)/2)
+    b = Operator('B', Symbol('t'), S.Half)
     inv = a.inv()
     f = Function('f')
     x = symbols('x')
@@ -513,7 +521,7 @@ def test_operator():
     op = OuterProduct(Ket(), Bra())
     assert str(a) == 'A'
     assert pretty(a) == 'A'
-    assert upretty(a) == u('A')
+    assert upretty(a) == u'A'
     assert latex(a) == 'A'
     sT(a, "Operator(Symbol('A'))")
     assert str(inv) == 'A**(-1)'
@@ -536,7 +544,7 @@ A  \
 """\
                     /d            \\\n\
 DifferentialOperator|--(f(x)),f(x)|\n\
-                    \dx           /\
+                    \\dx           /\
 """
     ucode_str = \
 u("""\
@@ -547,16 +555,16 @@ DifferentialOperator⎜──(f(x)),f(x)⎟\n\
     assert pretty(d) == ascii_str
     assert upretty(d) == ucode_str
     assert latex(d) == \
-        r'DifferentialOperator\left(\frac{d}{d x} f{\left (x \right )},f{\left (x \right )}\right)'
-    sT(d, "DifferentialOperator(Derivative(Function('f')(Symbol('x')), Symbol('x')),Function('f')(Symbol('x')))")
+        r'DifferentialOperator\left(\frac{d}{d x} f{\left(x \right)},f{\left(x \right)}\right)'
+    sT(d, "DifferentialOperator(Derivative(Function('f')(Symbol('x')), Tuple(Symbol('x'), Integer(1))),Function('f')(Symbol('x')))")
     assert str(b) == 'Operator(B,t,1/2)'
     assert pretty(b) == 'Operator(B,t,1/2)'
-    assert upretty(b) == u('Operator(B,t,1/2)')
+    assert upretty(b) == u'Operator(B,t,1/2)'
     assert latex(b) == r'Operator\left(B,t,\frac{1}{2}\right)'
     sT(b, "Operator(Symbol('B'),Symbol('t'),Rational(1, 2))")
     assert str(op) == '|psi><psi|'
     assert pretty(op) == '|psi><psi|'
-    assert upretty(op) == u('❘ψ⟩⟨ψ❘')
+    assert upretty(op) == u'❘ψ⟩⟨ψ❘'
     assert latex(op) == r'{\left|\psi\right\rangle }{\left\langle \psi\right|}'
     sT(op, "OuterProduct(Ket(Symbol('psi')),Bra(Symbol('psi')))")
 
@@ -565,7 +573,7 @@ def test_qexpr():
     q = QExpr('q')
     assert str(q) == 'q'
     assert pretty(q) == 'q'
-    assert upretty(q) == u('q')
+    assert upretty(q) == u'q'
     assert latex(q) == r'q'
     sT(q, "QExpr(Symbol('q'))")
 
@@ -575,12 +583,12 @@ def test_qubit():
     q2 = IntQubit(8)
     assert str(q1) == '|0101>'
     assert pretty(q1) == '|0101>'
-    assert upretty(q1) == u('❘0101⟩')
+    assert upretty(q1) == u'❘0101⟩'
     assert latex(q1) == r'{\left|0101\right\rangle }'
     sT(q1, "Qubit(Integer(0),Integer(1),Integer(0),Integer(1))")
     assert str(q2) == '|8>'
     assert pretty(q2) == '|8>'
-    assert upretty(q2) == u('❘8⟩')
+    assert upretty(q2) == u'❘8⟩'
     assert latex(q2) == r'{\left|8\right\rangle }'
     sT(q2, "IntQubit(8)")
 
@@ -643,41 +651,41 @@ J \n\
     sT(Jz, "JzOp(Symbol('J'))")
     assert str(ket) == '|1,0>'
     assert pretty(ket) == '|1,0>'
-    assert upretty(ket) == u('❘1,0⟩')
+    assert upretty(ket) == u'❘1,0⟩'
     assert latex(ket) == r'{\left|1,0\right\rangle }'
     sT(ket, "JzKet(Integer(1),Integer(0))")
     assert str(bra) == '<1,0|'
     assert pretty(bra) == '<1,0|'
-    assert upretty(bra) == u('⟨1,0❘')
+    assert upretty(bra) == u'⟨1,0❘'
     assert latex(bra) == r'{\left\langle 1,0\right|}'
     sT(bra, "JzBra(Integer(1),Integer(0))")
     assert str(cket) == '|1,0,j1=1,j2=2>'
     assert pretty(cket) == '|1,0,j1=1,j2=2>'
-    assert upretty(cket) == u('❘1,0,j₁=1,j₂=2⟩')
+    assert upretty(cket) == u'❘1,0,j₁=1,j₂=2⟩'
     assert latex(cket) == r'{\left|1,0,j_{1}=1,j_{2}=2\right\rangle }'
     sT(cket, "JzKetCoupled(Integer(1),Integer(0),Tuple(Integer(1), Integer(2)),Tuple(Tuple(Integer(1), Integer(2), Integer(1))))")
     assert str(cbra) == '<1,0,j1=1,j2=2|'
     assert pretty(cbra) == '<1,0,j1=1,j2=2|'
-    assert upretty(cbra) == u('⟨1,0,j₁=1,j₂=2❘')
+    assert upretty(cbra) == u'⟨1,0,j₁=1,j₂=2❘'
     assert latex(cbra) == r'{\left\langle 1,0,j_{1}=1,j_{2}=2\right|}'
     sT(cbra, "JzBraCoupled(Integer(1),Integer(0),Tuple(Integer(1), Integer(2)),Tuple(Tuple(Integer(1), Integer(2), Integer(1))))")
     assert str(cket_big) == '|1,0,j1=1,j2=2,j3=3,j(1,2)=3>'
     # TODO: Fix non-unicode pretty printing
     # i.e. j1,2 -> j(1,2)
     assert pretty(cket_big) == '|1,0,j1=1,j2=2,j3=3,j1,2=3>'
-    assert upretty(cket_big) == u('❘1,0,j₁=1,j₂=2,j₃=3,j₁,₂=3⟩')
+    assert upretty(cket_big) == u'❘1,0,j₁=1,j₂=2,j₃=3,j₁,₂=3⟩'
     assert latex(cket_big) == \
         r'{\left|1,0,j_{1}=1,j_{2}=2,j_{3}=3,j_{1,2}=3\right\rangle }'
     sT(cket_big, "JzKetCoupled(Integer(1),Integer(0),Tuple(Integer(1), Integer(2), Integer(3)),Tuple(Tuple(Integer(1), Integer(2), Integer(3)), Tuple(Integer(1), Integer(3), Integer(1))))")
     assert str(cbra_big) == '<1,0,j1=1,j2=2,j3=3,j(1,2)=3|'
-    assert pretty(cbra_big) == u('<1,0,j1=1,j2=2,j3=3,j1,2=3|')
-    assert upretty(cbra_big) == u('⟨1,0,j₁=1,j₂=2,j₃=3,j₁,₂=3❘')
+    assert pretty(cbra_big) == u'<1,0,j1=1,j2=2,j3=3,j1,2=3|'
+    assert upretty(cbra_big) == u'⟨1,0,j₁=1,j₂=2,j₃=3,j₁,₂=3❘'
     assert latex(cbra_big) == \
         r'{\left\langle 1,0,j_{1}=1,j_{2}=2,j_{3}=3,j_{1,2}=3\right|}'
     sT(cbra_big, "JzBraCoupled(Integer(1),Integer(0),Tuple(Integer(1), Integer(2), Integer(3)),Tuple(Tuple(Integer(1), Integer(2), Integer(3)), Tuple(Integer(1), Integer(3), Integer(1))))")
     assert str(rot) == 'R(1,2,3)'
     assert pretty(rot) == 'R (1,2,3)'
-    assert upretty(rot) == u('ℛ (1,2,3)')
+    assert upretty(rot) == u'ℛ (1,2,3)'
     assert latex(rot) == r'\mathcal{R}\left(1,2,3\right)'
     sT(rot, "Rotation(Integer(1),Integer(2),Integer(3))")
     assert str(bigd) == 'WignerD(1, 2, 3, 4, 5, 6)'
@@ -726,12 +734,12 @@ def test_state():
     tket = TimeDepKet()
     assert str(bra) == '<psi|'
     assert pretty(bra) == '<psi|'
-    assert upretty(bra) == u('⟨ψ❘')
+    assert upretty(bra) == u'⟨ψ❘'
     assert latex(bra) == r'{\left\langle \psi\right|}'
     sT(bra, "Bra(Symbol('psi'))")
     assert str(ket) == '|psi>'
     assert pretty(ket) == '|psi>'
-    assert upretty(ket) == u('❘ψ⟩')
+    assert upretty(ket) == u'❘ψ⟩'
     assert latex(ket) == r'{\left|\psi\right\rangle }'
     sT(ket, "Ket(Symbol('psi'))")
     assert str(bra_tall) == '<x/2|'
@@ -773,13 +781,13 @@ u("""\
     assert latex(ket_tall) == r'{\left|\frac{x}{2}\right\rangle }'
     sT(ket_tall, "Ket(Mul(Rational(1, 2), Symbol('x')))")
     assert str(tbra) == '<psi;t|'
-    assert pretty(tbra) == u('<psi;t|')
-    assert upretty(tbra) == u('⟨ψ;t❘')
+    assert pretty(tbra) == u'<psi;t|'
+    assert upretty(tbra) == u'⟨ψ;t❘'
     assert latex(tbra) == r'{\left\langle \psi;t\right|}'
     sT(tbra, "TimeDepBra(Symbol('psi'),Symbol('t'))")
     assert str(tket) == '|psi;t>'
     assert pretty(tket) == '|psi;t>'
-    assert upretty(tket) == u('❘ψ;t⟩')
+    assert upretty(tket) == u'❘ψ;t⟩'
     assert latex(tket) == r'{\left|\psi;t\right\rangle }'
     sT(tket, "TimeDepKet(Symbol('psi'),Symbol('t'))")
 
@@ -788,7 +796,7 @@ def test_tensorproduct():
     tp = TensorProduct(JzKet(1, 1), JzKet(1, 0))
     assert str(tp) == '|1,1>x|1,0>'
     assert pretty(tp) == '|1,1>x |1,0>'
-    assert upretty(tp) == u('❘1,1⟩⨂ ❘1,0⟩')
+    assert upretty(tp) == u'❘1,1⟩⨂ ❘1,0⟩'
     assert latex(tp) == \
         r'{{\left|1,1\right\rangle }}\otimes {{\left|1,0\right\rangle }}'
     sT(tp, "TensorProduct(JzKet(Integer(1),Integer(1)), JzKet(Integer(1),Integer(0)))")
@@ -809,7 +817,7 @@ def test_big_expr():
                  |/                                   +\\         |                                 \n\
     2  / +    +\\ <|                    /d            \\ |   +    +>                                 \n\
 /J \\ x \\A  + B /*||DifferentialOperator|--(f(x)),f(x)| | ,A  + B |*(<1,0| + <1,1|)*(|0,0> + |1,-1>)\n\
-\\ z/             \\\\                    \dx           / /         /                                 \
+\\ z/             \\\\                    \\dx           / /         /                                 \
 """
     ucode_str = \
 u("""\
@@ -822,8 +830,8 @@ u("""\
     assert pretty(e1) == ascii_str
     assert upretty(e1) == ucode_str
     assert latex(e1) == \
-        r'{J_z^{2}}\otimes \left({A^{\dag} + B^{\dag}}\right) \left\{\left(DifferentialOperator\left(\frac{d}{d x} f{\left (x \right )},f{\left (x \right )}\right)^{\dag}\right)^{3},A^{\dag} + B^{\dag}\right\} \left({\left\langle 1,0\right|} + {\left\langle 1,1\right|}\right) \left({\left|0,0\right\rangle } + {\left|1,-1\right\rangle }\right)'
-    sT(e1, "Mul(TensorProduct(Pow(JzOp(Symbol('J')), Integer(2)), Add(Dagger(Operator(Symbol('A'))), Dagger(Operator(Symbol('B'))))), AntiCommutator(Pow(Dagger(DifferentialOperator(Derivative(Function('f')(Symbol('x')), Symbol('x')),Function('f')(Symbol('x')))), Integer(3)),Add(Dagger(Operator(Symbol('A'))), Dagger(Operator(Symbol('B'))))), Add(JzBra(Integer(1),Integer(0)), JzBra(Integer(1),Integer(1))), Add(JzKet(Integer(0),Integer(0)), JzKet(Integer(1),Integer(-1))))")
+        r'{J_z^{2}}\otimes \left({A^{\dagger} + B^{\dagger}}\right) \left\{\left(DifferentialOperator\left(\frac{d}{d x} f{\left(x \right)},f{\left(x \right)}\right)^{\dagger}\right)^{3},A^{\dagger} + B^{\dagger}\right\} \left({\left\langle 1,0\right|} + {\left\langle 1,1\right|}\right) \left({\left|0,0\right\rangle } + {\left|1,-1\right\rangle }\right)'
+    sT(e1, "Mul(TensorProduct(Pow(JzOp(Symbol('J')), Integer(2)), Add(Dagger(Operator(Symbol('A'))), Dagger(Operator(Symbol('B'))))), AntiCommutator(Pow(Dagger(DifferentialOperator(Derivative(Function('f')(Symbol('x')), Tuple(Symbol('x'), Integer(1))),Function('f')(Symbol('x')))), Integer(3)),Add(Dagger(Operator(Symbol('A'))), Dagger(Operator(Symbol('B'))))), Add(JzBra(Integer(1),Integer(0)), JzBra(Integer(1),Integer(1))), Add(JzKet(Integer(0),Integer(0)), JzKet(Integer(1),Integer(-1))))")
     assert str(e2) == '[Jz**2,A + B]*{E**(-2),Dagger(D)*Dagger(C)}*[J2,Jz]'
     ascii_str = \
 """\
@@ -840,7 +848,7 @@ u("""\
     assert pretty(e2) == ascii_str
     assert upretty(e2) == ucode_str
     assert latex(e2) == \
-        r'\left[J_z^{2},A + B\right] \left\{E^{-2},D^{\dag} C^{\dag}\right\} \left[J^2,J_z\right]'
+        r'\left[J_z^{2},A + B\right] \left\{E^{-2},D^{\dagger} C^{\dagger}\right\} \left[J^2,J_z\right]'
     sT(e2, "Mul(Commutator(Pow(JzOp(Symbol('J')), Integer(2)),Add(Operator(Symbol('A')), Operator(Symbol('B')))), AntiCommutator(Pow(Operator(Symbol('E')), Integer(-2)),Mul(Dagger(Operator(Symbol('D'))), Dagger(Operator(Symbol('C'))))), Commutator(J2Op(Symbol('J')),JzOp(Symbol('J'))))")
     assert str(e3) == \
         "Wigner3j(1, 2, 3, 4, 5, 6)*[Dagger(B) + A,C + D]x(-J2 + Jz)*|1,0><1,1|*(|1,0,j1=1,j2=1> + |1,1,j1=1,j2=1>)x|1,-1,j1=1,j2=1>"
@@ -861,13 +869,13 @@ u("""\
     assert pretty(e3) == ascii_str
     assert upretty(e3) == ucode_str
     assert latex(e3) == \
-        r'\left(\begin{array}{ccc} 1 & 3 & 5 \\ 2 & 4 & 6 \end{array}\right) {\left[B^{\dag} + A,C + D\right]}\otimes \left({- J^2 + J_z}\right) {\left|1,0\right\rangle }{\left\langle 1,1\right|} \left({{\left|1,0,j_{1}=1,j_{2}=1\right\rangle } + {\left|1,1,j_{1}=1,j_{2}=1\right\rangle }}\right)\otimes {{\left|1,-1,j_{1}=1,j_{2}=1\right\rangle }}'
+        r'\left(\begin{array}{ccc} 1 & 3 & 5 \\ 2 & 4 & 6 \end{array}\right) {\left[B^{\dagger} + A,C + D\right]}\otimes \left({- J^2 + J_z}\right) {\left|1,0\right\rangle }{\left\langle 1,1\right|} \left({{\left|1,0,j_{1}=1,j_{2}=1\right\rangle } + {\left|1,1,j_{1}=1,j_{2}=1\right\rangle }}\right)\otimes {{\left|1,-1,j_{1}=1,j_{2}=1\right\rangle }}'
     sT(e3, "Mul(Wigner3j(Integer(1), Integer(2), Integer(3), Integer(4), Integer(5), Integer(6)), TensorProduct(Commutator(Add(Dagger(Operator(Symbol('B'))), Operator(Symbol('A'))),Add(Operator(Symbol('C')), Operator(Symbol('D')))), Add(Mul(Integer(-1), J2Op(Symbol('J'))), JzOp(Symbol('J')))), OuterProduct(JzKet(Integer(1),Integer(0)),JzBra(Integer(1),Integer(1))), TensorProduct(Add(JzKetCoupled(Integer(1),Integer(0),Tuple(Integer(1), Integer(1)),Tuple(Tuple(Integer(1), Integer(2), Integer(1)))), JzKetCoupled(Integer(1),Integer(1),Tuple(Integer(1), Integer(1)),Tuple(Tuple(Integer(1), Integer(2), Integer(1))))), JzKetCoupled(Integer(1),Integer(-1),Tuple(Integer(1), Integer(1)),Tuple(Tuple(Integer(1), Integer(2), Integer(1))))))")
-    assert str(e4) == '(C(1)*C(2)+F**2)*(L2([0, oo))+H)'
+    assert str(e4) == '(C(1)*C(2)+F**2)*(L2(Interval(0, oo))+H)'
     ascii_str = \
 """\
 // 1    2\\    x2\\   / 2    \\\n\
-\\\\C  x C / + F  / x \L  + H/\
+\\\\C  x C / + F  / x \\L  + H/\
 """
     ucode_str = \
 u("""\
@@ -878,10 +886,10 @@ u("""\
     assert upretty(e4) == ucode_str
     assert latex(e4) == \
         r'\left(\left(\mathcal{C}^{1}\otimes \mathcal{C}^{2}\right)\oplus {\mathcal{F}}^{\otimes 2}\right)\otimes \left({\mathcal{L}^2}\left( \left[0, \infty\right) \right)\oplus \mathcal{H}\right)'
-    sT(e4, "TensorProductHilbertSpace((DirectSumHilbertSpace(TensorProductHilbertSpace(ComplexSpace(Integer(1)),ComplexSpace(Integer(2))),TensorPowerHilbertSpace(FockSpace(),Integer(2)))),(DirectSumHilbertSpace(L2(Interval(Integer(0), oo, S.false, S.true)),HilbertSpace())))")
+    sT(e4, "TensorProductHilbertSpace((DirectSumHilbertSpace(TensorProductHilbertSpace(ComplexSpace(Integer(1)),ComplexSpace(Integer(2))),TensorPowerHilbertSpace(FockSpace(),Integer(2)))),(DirectSumHilbertSpace(L2(Interval(Integer(0), oo, false, true)),HilbertSpace())))")
 
 
 def _test_sho1d():
     ad = RaisingOp('a')
-    assert pretty(ad) == u(' \N{DAGGER}\na ')
-    assert latex(ad) == 'a^{\\dag}'
+    assert pretty(ad) == u' \N{DAGGER}\na '
+    assert latex(ad) == 'a^{\\dagger}'
